"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ajv_1 = require("../framework/ajv");
const util_1 = require("./util");
const ono_1 = require("ono");
const parameters_parse_1 = require("./schemas/parameters.parse");
const parameters_transform_1 = require("./schemas/parameters.transform");
const body_parse_1 = require("./schemas/body.parse");
class RequestValidator {
    constructor(apiDocs, options = {}) {
        this._middlewareCache = {};
        this._requestOpts = {};
        this._middlewareCache = {};
        this._apiDocs = apiDocs;
        this._requestOpts.allowUnknownQueryParameters =
            options.allowUnknownQueryParameters;
        this.ajv = ajv_1.createRequestAjv(apiDocs, options);
    }
    validate(req, res, next) {
        var _a;
        if (!req.openapi) {
            // this path was not found in open api and
            // this path is not defined under an openapi base path
            // skip it
            return next();
        }
        const openapi = req.openapi;
        const path = openapi.expressRoute;
        if (!path) {
            throw util_1.validationError(404, req.path, 'not found');
        }
        const pathSchema = openapi.schema;
        if (!pathSchema) {
            throw util_1.validationError(405, req.path, `${req.method} method not allowed`);
        }
        // cache middleware by combining method, path, and contentType
        const contentType = util_1.ContentType.from(req);
        const contentTypeKey = (_a = contentType.equivalents()[0], (_a !== null && _a !== void 0 ? _a : 'not_provided'));
        const key = `${req.method}-${req.originalUrl}-${contentTypeKey}`;
        if (!this._middlewareCache[key]) {
            const middleware = this.buildMiddleware(path, pathSchema, contentType);
            this._middlewareCache[key] = middleware;
        }
        return this._middlewareCache[key](req, res, next);
    }
    buildMiddleware(path, pathSchema, contentType) {
        const paramSchemaParser = new parameters_parse_1.ParametersSchemaParser(this._apiDocs);
        const parameters = paramSchemaParser.parse(path, pathSchema.parameters);
        const securityQueryParam = Security.queryParam(this._apiDocs, pathSchema);
        const bodySchemaParser = new body_parse_1.BodySchemaParser(this.ajv, this._apiDocs);
        // TODO bodyParser.parse should return OpenAPIV3.SchemaObject instead of BodySchema
        const body = (bodySchemaParser.parse(path, pathSchema, contentType));
        ;
        const required = body.required ? ['body'] : [];
        // $schema: "http://json-schema.org/draft-04/schema#",
        const schema = {
            required: ['query', 'headers', 'params'].concat(required),
            properties: Object.assign(Object.assign({}, parameters.schema), { body: body }),
        };
        const validator = this.ajv.compile(schema);
        return (req, res, next) => {
            var _a;
            const parametersRequest = new parameters_transform_1.ParametersTransform(parameters, schema);
            parametersRequest.applyExplodedJsonTransform(req);
            parametersRequest.applyExplodedJsonArrayTransform(req);
            if (!this._requestOpts.allowUnknownQueryParameters) {
                this.rejectUnknownQueryParams(req.query, schema.properties.query, securityQueryParam);
            }
            parametersRequest.applyPathTransform(req);
            parametersRequest.applyJsonTransform(req);
            parametersRequest.applyJsonArrayTransform(req);
            const cookies = req.cookies
                ? Object.assign(Object.assign({}, req.cookies), req.signedCookies) : undefined;
            const valid = validator(Object.assign(Object.assign({}, req), { cookies }));
            if (valid) {
                next();
            }
            else {
                const errors = util_1.augmentAjvErrors([...(_a = validator.errors, (_a !== null && _a !== void 0 ? _a : []))]);
                const err = util_1.ajvErrorsToValidatorError(400, errors);
                const message = this.ajv.errorsText(errors, { dataVar: 'request' });
                throw ono_1.default(err, message);
            }
        };
    }
    rejectUnknownQueryParams(query, schema, whiteList = []) {
        if (!schema.properties)
            return;
        const knownQueryParams = new Set(Object.keys(schema.properties));
        whiteList.forEach(item => knownQueryParams.add(item));
        const queryParams = Object.keys(query);
        for (const q of queryParams) {
            if (!knownQueryParams.has(q)) {
                throw util_1.validationError(400, `.query.${q}`, `Unknown query parameter ${q}`);
            }
        }
    }
}
exports.RequestValidator = RequestValidator;
class Security {
    static queryParam(apiDocs, schema) {
        var _a;
        const hasPathSecurity = schema.hasOwnProperty('security') && schema.security.length > 0;
        const hasRootSecurity = apiDocs.hasOwnProperty('security') && apiDocs.security.length > 0;
        let usedSecuritySchema = [];
        if (hasPathSecurity) {
            usedSecuritySchema = schema.security;
        }
        else if (hasRootSecurity) {
            // if no security schema for the path, use top-level security schema
            usedSecuritySchema = apiDocs.security;
        }
        const securityQueryParameter = this.getSecurityQueryParams(usedSecuritySchema, (_a = apiDocs.components) === null || _a === void 0 ? void 0 : _a.securitySchemes);
        return securityQueryParameter;
    }
    static getSecurityQueryParams(usedSecuritySchema, securitySchema) {
        return usedSecuritySchema && securitySchema
            ? usedSecuritySchema
                .filter(obj => Object.entries(obj).length !== 0)
                .map(sec => {
                const securityKey = Object.keys(sec)[0];
                return securitySchema[securityKey];
            })
                .filter(sec => { var _a; return ((_a = sec) === null || _a === void 0 ? void 0 : _a.in) === 'query'; })
                .map(sec => sec.name)
            : [];
    }
}
//# sourceMappingURL=openapi.request.validator.js.map